package org.springframework.webflow.samples.phonebook.webflow;

import java.io.ByteArrayOutputStream;
import java.io.ObjectOutputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.webflow.definition.StateDefinition;
import org.springframework.webflow.execution.FlowExecutionListenerAdapter;
import org.springframework.webflow.execution.RequestContext;
import org.springframework.webflow.execution.ViewSelection;
import org.springframework.webflow.execution.support.FlowExecutionRedirect;

public class FlowExecutionMonitor extends FlowExecutionListenerAdapter {

	protected final Log logger = LogFactory.getLog(getClass());
	
	@Override
	public void requestSubmitted(RequestContext context) {
		StringBuilder sb = new StringBuilder("\nWeb Flow request for ");
		sb.append(context.getFlowExecutionContext().getDefinition().getId());
		logger.debug(sb.toString());
	}
	
	@Override
	public void requestProcessed(RequestContext context) {
		StringBuilder sb = new StringBuilder();
		sb.append("Web Flow finish request for ");
		sb.append(context.getFlowExecutionContext().getDefinition().getId());
		logger.debug(sb.toString());
	}

	@Override
	public void paused(RequestContext context, ViewSelection selectedView) {
		StringBuilder sb = new StringBuilder();
		sb.append("Paused to render view ").append(context.getCurrentState().getId());
		sb.append(" in ").append(context.getCurrentState().getOwner().getId());
		if (selectedView instanceof FlowExecutionRedirect) {
			sb.append(" (redirect via GET)");
		}
		logger.debug(sb.toString());
		reportWebFlowState(context);
	}

	@Override
	public void stateEntered(RequestContext context, StateDefinition previousState, StateDefinition newState) {
		StringBuilder sb = new StringBuilder();
		sb.append("Entered state ").append(newState.getId());
		sb.append(" in ").append(newState.getOwner().getId());
		logger.debug(sb.toString());
		reportWebFlowState(context);
	}

	private void reportWebFlowState(RequestContext context) {
		StringBuilder sb;
		sb = new StringBuilder();

		try {
			HashMap emptyMap = new HashMap();
			int emptyMapSize = getSerializedSize(emptyMap);

			Map asMap = context.getFlashScope().asMap();
			sb.append("flashScope=").append(getSerializedSize(asMap)-emptyMapSize).append(" (");
			appendMapContent(sb, asMap);
			sb.append("), ");
			
			asMap = context.getRequestScope().asMap(); 
			sb.append("requestScope=").append(getSerializedSize(asMap)-emptyMapSize).append(" (");
			appendMapContent(sb, asMap);
			sb.append("), ");
			
			asMap = context.getFlowScope().asMap();
			sb.append("flowScope=").append(getSerializedSize(asMap)-emptyMapSize).append(" (");
			appendMapContent(sb, asMap);
			sb.append("), ");
			
			asMap = context.getConversationScope().asMap();
			sb.append("conversationScope=").append(getSerializedSize(asMap)-emptyMapSize).append(" (");
			appendMapContent(sb, asMap);
			sb.append(") ");
		
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		logger.debug(sb.toString());
	}

	private void appendMapContent(StringBuilder sb, Map asMap) {
		
		if (asMap.keySet().isEmpty()) {
			return;
		}
		int i=0;
		Iterator itr = asMap.keySet().iterator();
		while (itr.hasNext()) {
			Object key = itr.next();
			if (i > 0) {
				sb.append(", ");
			}
			sb.append("'").append(key).append("'");
			i++;
		}
	}

	private int getSerializedSize(Object obj) throws Exception {
		
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		ObjectOutputStream oout = new ObjectOutputStream(bout);
		oout.writeObject(obj);
		oout.flush();
		int objSize = bout.toByteArray().length;
		return objSize;
	}	
}
